<?php

namespace App\Service\Oa;


use App\Model\Employee;
use App\Model\Oa;
use App\Model\OaOrder;
use App\Model\OaOrderSub;
use App\Model\OaOrderSubEmployee;
use App\Model\OaSub;
use Illuminate\Support\Facades\DB;
use App\Service\Service;

/**
 * Oa相关
 * @package App\Models
 */
class OaService extends Service
{

    public function oaList($data)
    {

        $list = Oa::from('oa as o')
            ->leftjoin('sys_menu as s', 's.id', 'o.menu_id')
            ->select('o.id', 'o.menu_id', 'o.crt_time', 's.title')
            ->where('o.del_time', 0);

        $list = $this->limit($list, '', $data);


        return [true, $list];

    }


    public function oaDetail($data)
    {

        $id = $data['id'];

//        if($this->isEmpty($data,'title')) return [201,'名称不存在！'];
        $detail = OaSub::from('oa_sub as os')
            ->leftJoin('employee as e', 'e.id', 'os.employee_id')
            ->where('oa_id', $id)
            ->where('os.del_time', 0)
            ->select('os.sort', 'os.type', 'os.employee_id', 'os.remark', 'e.emp_name','os.type')
            ->orderBy('os.sort', 'asc')->get()->toArray();
        $first = Oa::from('oa as o')
            ->leftjoin('sys_menu as s', 's.id', 'o.menu_id')
            ->select('o.id', 'o.menu_id', 'o.crt_time', 's.title','o.type')
            ->where('o.del_time', 0)->where('o.id', $id)->first()->toArray();

        $first['detail'] = $detail;

        return [true, $first];

    }

    public function oaAdd($data)
    {

        $first = Oa::where('del_time', 0)->where('menu_id', $data['menu_id']);
        if(isset($data['type'])) $first = $first->where('type',$data['type']);
        $first = $first->first();

        if ($first) return [false, '该菜单oa已存在!'];
        $oa = new Oa();


        try {
            DB::beginTransaction();
            $oa->menu_id = $data['menu_id'];
            $oa->type = $data['type'];
            $oa->save();
            $id = $oa->id;
            $insert = [];
            foreach ($data['detail'] as $v) {
                foreach ($v['employee_id'] as $e_id) {
                    $insert[] = [
                        'oa_id' => $id,
                        'remark' => $v['remark'],
                        'employee_id' => $e_id,
                        'sort' => $v['sort'],
                        'type' => $v['type'],
                        'crt_time' => time(),
                        'upd_time' => time(),
                    ];
                }

            }
            OaSub::insert($insert);
            DB::commit();
        } catch (\Exception $e) {
            DB::rollBack();
            return [false, $e->getLine() . ':' . $e->getMessage()];
        }

        return [true, '保存成功！'];

    }


    public function oaEdit($data)
    {
        $first = Oa::where('del_time', 0)->where('id', '<>', $data['id'])->where('menu_id', $data['menu_id']);
        if(isset($data['type'])) $first = $first->where('type',$data['type']);
        $first = $first->first();

        if ($first) return [false, '该菜单oa已存在!'];


        try {
            DB::beginTransaction();
            $oa = Oa::where('id', $data['id'])->first();
            $oa->menu_id = $data['menu_id'];
            $oa->type = $data['type'];
            $oa->save();
            $id = $oa->id;
            $insert = [];
            OaSub::where('oa_id', $id)->update([
                'del_time' => time()
            ]);
            foreach ($data['detail'] as $v) {
                foreach ($v['employee_id'] as $e_id) {
                    $insert[] = [
                        'oa_id' => $id,
                        'remark' => $v['remark'],
                        'employee_id' => $e_id,
                        'sort' => $v['sort'],
                        'type' => $v['type'],
                        'crt_time' => time(),
                        'upd_time' => time(),
                    ];
                }
            }
            OaSub::insert($insert);
            DB::commit();
        } catch (\Exception $e) {
            DB::rollBack();
            return [false, $e->getMessage()];
        }

        return [true, '更新成功'];
    }

    public function oaDel($data)
    {
        $id = $data['id'];
        Oa::where('id', $id)->update([
            'del_time' => time(),
        ]);

        OaSub::where('oa_id', $id)->update([
            'del_time' => time(),
        ]);

        return [true,'删除成功'];
    }

    public function oaSubType($data){
        $service = new CheckService();
        $list = $service->map;

        $return = [];
        foreach ($list as $k=>$v){
            $return[] = [
                'id' => $k,
                'title' => $v,
            ];
        }

        return [true,$return];
    }


    public function createOaOrder($data)
    {
        $menu_id = $data['menu_id'];
        $order_no = $data['order_no'];
        $type = isset($data['type']) ? $data['type'] : 0;

        $list = Oa::from('oa as o')
            ->leftJoin('oa_sub as os', 'os.oa_id', 'o.id')
            ->select('os.sort', 'os.type', 'os.employee_id')
            ->where('o.menu_id', $menu_id);
        if(!empty($type)) $list = $list->where('o.type',$type);
        $list = $list->orderBy('os.sort')
        ->get()->toArray();
        if (empty($list)) {
            $service =  new CheckService();
            $service->createRecordAndInventory([
                'order_number' => $order_no,
                'type' => 1,
                'opt_case' => $type,
            ]);
            return [200, ''];
        } //pending
        $time = time();
        try {
            DB::beginTransaction();
            $order = new OaOrder();
            $order->menu_id = $menu_id;
            $order->order_no = $order_no;
            $order->type = $type;
            $order->save();
            $id = $order->id;
            $insert = [];
            $sub_insert = [];
            $is_sort = [];
            foreach ($list as $v) {
                if(!in_array($v['sort'],$is_sort)){
                    $is_sort[] = $v['sort'];
                    $sub_insert[] = [
                        'oa_order_id' => $id,
                        'crt_time' => $time,
                        'upd_time' => $time,
                        'sort' => $v['sort'],
                        'type' => $v['type'],
                    ];
                }
                $insert[] = [
                    'oa_order_id' => $id,
                    'crt_time' => $time,
                    'upd_time' => $time,
                    'sort' => $v['sort'],
                    'type' => $v['type'],
                    'employee_id' => $v['employee_id'],
                    'remark' => '',
                ];
            }

            OaOrderSubEmployee::insert($insert);
            OaOrderSub::insert($sub_insert);


            DB::commit();
            return [201, '']; //success
        } catch (\Exception $e) {
            DB::rollBack();
            return [false, $e->getMessage()];
        }


    }

    public function oaOrderList($data){
        $list = OaOrder::from('oa_order as o')
            ->leftJoin('sys_menu as s','s.id','o.menu_id')
            ->select('o.order_no','o.id','o.crt_time','o.upd_time','o.state','s.title','s.uri');

        $list = $this->limit($list,'',$data);

        return [true,$list];
    }

    public function oaOrderDetail($data,$user){

        $id = $data['id'];
        $first = OaOrder::from('oa_order as o')
            ->leftJoin('sys_menu as s','s.id','o.menu_id')
            ->select('o.menu_id','o.param','o.crt_time','o.upd_time','o.state','o.order_no','s.title as menu_title','s.uri')->first()->toArray();
        $employee_list = OaOrderSub::from('oa_order_sub_employee as s')
            ->leftJoin('employee as e','e.id','s.employee_id')
            ->where('s.oa_order_id',$id)->select('s.sort','s.type','s.employee_id','s.remark','e.emp_name','s.state','s.crt_time','s.upd_time')->orderBy('s.sort','asc')->get()->toArray();
        $employee_key_list = [];
        foreach ($employee_list as $v){
            $employee_key_list[$v['sort']][] = $v;
        }
        $detail = OaOrderSub::where('oa_order_id',$id)->select('*')->orderBy('sort','asc')->get()->toArray();
        foreach ($detail as &$v){
            $v['list'] = $employee_key_list[$v['sort']];
        }
        $first['detail'] = $detail;

        return [true,$first];
    }

    public function oaCheck($data,$user){


        $service =  new CheckService();
        $id = $data['id'];
        $user_id = $user['id'];
//        $user_id = '230';
        $state = $data['state'];
        $remark = $data['remark'];

        $detail = OaOrder::where('id',$id)->first()->toArray();

        $oa_type = $detail['type'];
        $order_no = $detail['order_no'];
        $time = time();
        if($oa_type == 2) return [false,'订单无法重复审核'];
        $list = OaOrderSub::where('oa_order_id',$id)->orderBy('sort','asc')->select('*')->get()->toArray();
        $sub_list = OaOrderSubEmployee::where('oa_order_id',$id)->orderBy('sort','asc')->select('*')->get()->toArray();
        $employee_name = Employee::where('id',$user_id)->value('emp_name');
        list($unoa_sort_list,$complete_sort,$employee_list,$max_sort) = $this->dealBeforeOaUser($list,$sub_list);
        if(!isset($employee_list[$user_id])) return [false,'该用户无法审核'];

        $sort = $employee_list[$user_id];

        if($state == 1) {

            if($sort <= $complete_sort) return [false,'用户无法审核'];

            $update_employee = [];
            $completing_sort = [];
            foreach ($unoa_sort_list as $v){
                if($v['sort'] < $sort){
                    $update_employee[] = $v['employee_id'];
                    $completing_sort[] = $v['sort'];
                }
            }
            OaOrderSubEmployee::wherein('employee_id',$update_employee)->where('oa_order_id',$id)->update([
                'state' => 1,
                'remark' => $employee_name.'审核通过,自动成功',
                'upd_time' => $time,
            ]);

            OaOrderSub::wherein('sort',$completing_sort)->where('oa_order_id',$id)->update([
                'state' => 1,
                'upd_time' => $time,
            ]);

            OaOrderSubEmployee::where('employee_id',$user_id)->where('oa_order_id',$id)->update([
                'state' => 1,
                'remark' => $remark,
                'upd_time' => $time,
            ]);

            OaOrderSub::where('sort',$sort)->where('oa_order_id',$id)->update([
                'state' => 1,
                'upd_time' => $time,
            ]);

            if($sort == $max_sort){
                OaOrder::where('id',$id)->update([
                    'state' => 2,
                    'upd_time' => $time,
                ]);
                $service->createRecordAndInventory([
                    'order_number' => $detail['order_no'],
                    'type' => 1,
                    'opt_case' => $oa_type,
                ]);
            }else{
                OaOrder::where('id',$id)->update([
                    'state' => 1,
                    'upd_time' => $time,
                ]);
            }

        }else{
            if($sort == 1){
                OaOrder::where('id',$id)->update([
                    'del_time' => $time,
                    'upd_time' => $time,
                ]);

                OaOrderSubEmployee::where('oa_order_id',$id)->update([
                    'del_time' => $time,
                    'upd_time' => $time,
                ]);
                OaOrderSub::where('oa_order_id',$id)->update([
                    'del_time' => $time,
                    'upd_time' => $time,
                ]);
                $service->createRecordAndInventory([
                    'order_number' => $detail['order_no'],
                    'type' => 2,
                    'opt_case' => $oa_type,
                ]);

            }
            $before_sort = $sort-1;
            OaOrderSubEmployee::where('oa_order_id',$id)->where('sort',$before_sort)->update([
                'state' => 2,
                'remark' => $remark,
            ]);
            OaOrderSub::where('oa_order_id',$id)->where('sort',$before_sort)->update([
                'state' => 2,
                'upd_time' => $time,
            ]);
            if($before_sort == 1){
                OaOrder::where('id',$id)->update([
                    'state' => 0,
                    'upd_time' => $time,
                ]);
            }
        }

        return [true,'审核成功!'];
    }

    public function dealBeforeOaUser($list,$sub_list){

        $sub_key_list = [];
        $employee_list  = [];
        foreach ($sub_list as $v){
            $sub_key_list[$v['sort']][] = $v;

            if(!isset( $employee_list[$v['employee_id']])) $employee_list[$v['employee_id']] = $v['sort'];
            else {
                if($v['sort']> $employee_list[$v['employee_id']]) $employee_list[$v['employee_id']] = $v['sort'];
            }
        }

        $unoa_sort_list = [];

        $complete_sort = 0;
        foreach ($list as $v){
            if($v['state'] == 0||$v['state'] == 2){
                foreach ($sub_key_list[$v['sort']] as $vv){
                    $unoa_sort_list[] = [
                        'employee_id' => $vv['employee_id'],
                        'sort' => $v['sort'],
                    ];
                }

            }
            if($v['state'] == 1){

                    $complete_sort = $v['sort'];

            }

            $max_sort = $v['sort'];
        }

        return [$unoa_sort_list,$complete_sort,$employee_list,$max_sort];
    }

}
