<?php

namespace App\Service;

use App\Model\BasicType;
use App\Model\Employee;
use App\Model\Product;
use App\Model\ProductActivity;
use App\Model\ProductActivityPrice;
use Illuminate\Support\Facades\DB;

class ProductActivityService extends Service
{
    public function productEdit($data,$user){
        list($status,$msg) = $this->productRule($data, $user, false);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = ProductActivity::where('id',$data['id'])->first();
            $model->title = $data['title'] ?? "";
            $model->start_time = $data['start_time'] ?? 0;
            $model->end_time = $data['end_time'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->save();

            $time = time();

            ProductActivityPrice::where('product_activity_id',$data['id'])
                ->where('del_time',0)
                ->update(['del_time' => $time]);

            if(! empty($data['product'])){
                $insert = [];
                foreach ($data['product'] as $value){
                    $insert[] = [
                        'product_activity_id' => $data['id'],
                        'product_id' => $value['product_id'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'crt_time' => $time,
                        'start_time' => $data['start_time'] ?? 0,
                        'end_time' => $data['end_time'] ?? 0,
                    ];
                }
                ProductActivityPrice::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function productAdd($data,$user){
        list($status,$msg) = $this->productRule($data, $user);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = new ProductActivity();
            $model->title = $data['title'] ?? "";
            $model->start_time = $data['start_time'] ?? 0;
            $model->end_time = $data['end_time'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->crt_id = $user['id'];
            $model->depart_id = $data['depart_id'] ?? 0;
            $model->top_depart_id = $data['top_depart_id'] ?? 0;
            $model->save();

            $time = time();

            if(! empty($data['product'])){
                $insert = [];
                foreach ($data['product'] as $value){
                    $insert[] = [
                        'product_activity_id' => $model->id,
                        'product_id' =>  $value['product_id'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'crt_time' => $time,
                        'start_time' => $data['start_time'] ?? 0,
                        'end_time' => $data['end_time'] ?? 0,
                    ];
                }
                ProductActivityPrice::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function productDel($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];

        try {
            DB::beginTransaction();
            $time = time();

            ProductActivity::where('del_time',0)->where('id',$data['id'])->update(['del_time' => $time]);

            ProductActivityPrice::where('product_activity_id',$data['id'])
                ->where('del_time',0)
                ->update(['del_time' => $time]);

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function productDetail($data,$user){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];
        $customer = ProductActivity::where('del_time',0)
            ->where('id',$data['id'])
            ->first();
        if(empty($customer)) return [false,'活动不存在或已被删除'];
        $customer = $customer->toArray();

        $detail = ProductActivityPrice::where('del_time',0)
            ->where('product_activity_id',$data['id'])
            ->get()->toArray();
        $title_map = BasicType::whereIn('id',array_column($detail,'basic_type_id'))
            ->pluck('title','id')
            ->toArray();
        $customer['product'] = [];

        //产品字典
        $map = (new ProductService())->getProductDetail(array_unique(array_column($detail,'product_id')));

        foreach ($detail as $value){
            $basic_tmp = [
                'basic_type_id' => $value['basic_type_id'],
                'basic_type_title' => $title_map[$value['basic_type_id']] ?? '',
                'price' => $value['price'],
            ];
            if(! isset($customer['product'][$value['product_id']])){
                $tmp = $map[$value['product_id']] ?? [];
                $tmp['activity'][] = $basic_tmp;
                $customer['product'][$value['product_id']] = $tmp;
            }else{
                $customer['product'][$value['product_id']]['activity'][] = $basic_tmp;
            }
        }

        $customer['product'] = array_values($customer['product']);
        $customer['crt_name'] = Employee::where('id',$customer['crt_id'])->value('emp_name');
        $customer['crt_time'] = $customer['crt_time'] ? date("Y-m-d H:i:s",$customer['crt_time']): '';

        return [true, $customer];
    }

    public function productList($data,$user){
        $model = ProductActivity::TopClear($user,$data);
        $model = $model->where('del_time',0)
            ->select('id','title','crt_id','mark','start_time','end_time','crt_time')
            ->orderby('id', 'desc');

        if(! empty($data['title'])) $model->where('title', 'LIKE', '%'.$data['title'].'%');
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->where('crt_time','>=',$return[0]);
            $model->where('crt_time','<=',$return[1]);
        }
        if(! empty($data['activity_time'][0]) && ! empty($data['activity_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['activity_time']);
            $model->where('start_time','>=',$return[0]);
            $model->where('end_time','<=',$return[1]);
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillData($list,$user);

        return [true, $list];
    }

    public function productRule(&$data, $user, $is_add = true){
        if(empty($data['title'])) return [false,'活动名称名称不能为空'];
        if(empty($data['activity_time'][0]) || empty($data['activity_time'][1])) return [false,'请填写活动时间范围'];
        $data['start_time'] = $this->changeDateToDateMin($data['activity_time'][0]);
        $data['end_time'] = $this->changeDateToDateMin($data['activity_time'][1]);
        if($is_add){
            $product = ProductActivityPrice::where('del_time',0)
                ->where('start_time', '<=', $data['end_time'])
                ->where('end_time', '>=', $data['start_time'])
                ->select('product_id')
                ->get()->toArray();
            $product = array_column($product,'product_id');
        }else{
            if(empty($data['id'])) return [false,'ID不能为空'];
            $product = ProductActivityPrice::where('del_time',0)
                ->where('product_activity_id','<>',$data['id'])
                ->where('start_time', '<=', $data['end_time'])
                ->where('end_time', '>=', $data['start_time'])
                ->select('product_id')
                ->get()->toArray();
            $product = array_column($product,'product_id');
        }
        if(! empty($data['product'])){
            $start_time = date("Y-m-d H:i",$data['start_time']);
            $end_time = date("Y-m-d H:i",$data['end_time']);
            $map = BasicType::whereIn('id',array_column($data['product'],'basic_type_id'))
                ->pluck('title','id')
                ->toArray();
            $map2 = Product::whereIn('id',array_column($data['product'],'product_id'))
                ->pluck('title','id')
                ->toArray();
            foreach ($data['product'] as $value){
                $pro = $map2[$value['product_id']] ?? "";
                if(in_array($value['product_id'], $product)) return [false,'产品：' . $pro . '在' . $start_time . '——' . $end_time . '已设置活动价格'];
                if(! empty($value['price'])) {
                    $tmp = $map[$value['basic_type_id']] ?? '';
                    $res = $this->checkNumber($value['price']);
                    if(! $res) return [false, $tmp . '请输入不超过两位小数并且大于0的数值'];
                }
            }
        }

        //所属部门 以及 顶级部门
        if(empty($data['depart_id'])) $data['depart_id'] = $this->getDepart($user);
        $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;

        return [true, $data];
    }

    public function fillData($data, $user){
        if(empty($data['data'])) return $data;

        $emp = Employee::whereIn('id',array_unique(array_column($data['data'],'crt_id')))
            ->pluck('emp_name','id')
            ->toArray();

        foreach ($data['data'] as $key => $value){
            $start_time = $value['start_time'] ? date("Y-m-d H:i",$value['start_time']) : '';
            $end_time = $value['end_time'] ? date("Y-m-d H:i",$value['end_time']) : '';
            $data['data'][$key]['activity_time'] = $start_time . '——' . $end_time;
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date('Y-m-d H:i:s',$value['crt_time']) : '';
            $data['data'][$key]['crt_name'] = $emp[$value['crt_id']] ?? '';
        }

        return $data;
    }
}
