<?php

namespace App\Service;

use App\Model\BasicType;
use App\Model\Employee;
use App\Model\PurchaseOrder;
use App\Model\PurchaseOrderInfo;
use App\Model\ReturnExchangeOrder;
use App\Model\ReturnExchangeOrderProductInfo;
use App\Model\SalesOrder;
use App\Model\SalesOrderProductInfo;
use App\Model\SeeRange;
use App\Model\Storehouse;
use Illuminate\Support\Facades\DB;

class ReturnExchangeOrderService extends Service
{
    public function edit($data,$user){
        list($status,$msg) = $this->orderRule($data,$user,false);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $material_model = ReturnExchangeOrder::where('order_number',$data['order_number'])->first();
            $material_model->data_id = $data['data_id'];
            $material_model->type = $data['type'];
            $material_model->difference_amount = $data['difference_amount'];
            $material_model->mark = $data['mark'];
            $material_model->storehouse_id = $data['storehouse_id'] ?? 0;
            $material_model->save();
            $time = time();

            ReturnExchangeOrderProductInfo::where('del_time',0)
                ->where('return_exchange_id',$data['id'])
                ->update(['del_time' => $time]);
            if(!empty($data['product_one'])){
                $sub = [];
                foreach ($data['product_one'] as $value){
                    $sub[] = [
                        'return_exchange_id' => $material_model->id,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                        'cost' => $value['cost'] ?? 0,
                        'retail_price' => $value['retail_price'] ?? 0,
                        'return_or_exchange' => ReturnExchangeOrderProductInfo::type_one,
                        'crt_time' => $time,
                        'storehouse_id' => $data['storehouse_id'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'final_amount' => $value['final_amount'] ?? 0,
                        'return_exchange_price' => $value['return_exchange_price'],
                    ];
                }
                ReturnExchangeOrderProductInfo::insert($sub);
            }

            if(!empty($data['product_two'])){
                $sub = [];
                foreach ($data['product_two'] as $value){
                    $sub[] = [
                        'return_exchange_id' => $material_model->id,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                        'cost' => $value['cost'] ?? 0,
                        'retail_price' => $value['retail_price'] ?? 0,
                        'return_or_exchange' => ReturnExchangeOrderProductInfo::type_two,
                        'crt_time' => $time,
                        'storehouse_id' => $data['storehouse_id'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                    ];
                }
                ReturnExchangeOrderProductInfo::insert($sub);
            }

            //锁定库存
            ProductInventoryService::changeLockNumber($user, $msg[0],$msg[1]);

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    public function add($data,$user){
        list($status,$msg) = $this->orderRule($data,$user);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $material_model = new ReturnExchangeOrder();
            $material_model->model_type = $data['model_type'];
            $material_model->order_number = $data['order_number'];
            $material_model->data_id = $data['data_id'];
            $material_model->type = $data['type'];
            $material_model->difference_amount = $data['difference_amount'];
            $material_model->mark = $data['mark'];
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->top_depart_id = $data['top_depart_id'] ?? 0;
            $material_model->crt_id = $user['id'];
            $material_model->storehouse_id = $data['storehouse_id'];
            $material_model->save();
            $time = time();

            if(!empty($data['product_one'])){
                $sub = [];
                foreach ($data['product_one'] as $value){
                    $sub[] = [
                        'return_exchange_id' => $material_model->id,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                        'cost' => $value['cost'] ?? 0,
                        'retail_price' => $value['retail_price'] ?? 0,
                        'return_or_exchange' => ReturnExchangeOrderProductInfo::type_one,
                        'crt_time' => $time,
                        'storehouse_id' => $data['storehouse_id'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'final_amount' => $value['final_amount'] ?? 0,
                        'return_exchange_price' => $value['return_exchange_price'],
                    ];
                }
                ReturnExchangeOrderProductInfo::insert($sub);
            }

            if(!empty($data['product_two'])){
                $sub = [];
                foreach ($data['product_two'] as $value){
                    $sub[] = [
                        'return_exchange_id' => $material_model->id,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                        'cost' => $value['cost'] ?? 0,
                        'retail_price' => $value['retail_price'] ?? 0,
                        'return_or_exchange' => ReturnExchangeOrderProductInfo::type_two,
                        'crt_time' => $time,
                        'storehouse_id' => $data['storehouse_id'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                    ];
                }
                ReturnExchangeOrderProductInfo::insert($sub);
            }

            //锁定库存
            ProductInventoryService::changeLockNumber($user, $msg[0],[]);

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    public function detail($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据'];

        $order = ReturnExchangeOrder::where('id',$data['id'])
            ->where('del_time',0)
            ->select('id','order_number','model_type','data_id','difference_amount','crt_id','mark','state','crt_time','storehouse_id','depart_id','top_depart_id','type')
            ->first();
        if(empty($order)) return [false, '退换货订单不存在或已被删除'];
        $order = $order->toArray();
        $order['storehouse_title'] = Storehouse::where('id',$order['storehouse_id'])->value('title');
        if($order['type'] == ReturnExchangeOrder::Order_type){
            $sales = SalesOrder::where('id',$order['data_id'])->value('order_number');
            $order['data_title'] = $sales;
        }elseif($order['type'] == ReturnExchangeOrder::Order_type2){
            $sales = PurchaseOrder::where('id',$order['data_id'])->value('order_number');
            $order['data_title'] = $sales;
        }

        $order['model_type_title'] = ReturnExchangeOrder::$model_type_name[$order['model_type']] ?? "";
        $order['state_title'] = ReturnExchangeOrder::$state[$order['state']] ?? '';

        $order['product_one'] = $order['product_two'] = $order['depart'] = $order['employee'] = [];
        $info = ReturnExchangeOrderProductInfo::where('del_time',0)
            ->where('return_exchange_id',$data['id'])
            ->get()->toArray();
        $basic_price = BasicType::whereIn('id',array_unique(array_column($info,'basic_type_id')))->pluck('title','id')->toArray();
        $map = (new ProductService())->getProductDetail(array_column($info,'product_id'));
        foreach ($info as $value){
            $tmp = $map[$value['product_id']] ?? [];
            $value['title'] = $tmp['title'] ?? "";
            $value['code'] = $tmp['code'] ?? "";
            $value['size'] = $tmp['size'] ?? "";
            $value['unit'] = $tmp['unit'] ?? "";
            $value['bar_code'] = $tmp['bar_code'] ?? "";
            $value['basic_type_title'] = $basic_price[$value['basic_type_id']] ?? "";
            if($value['return_or_exchange'] == ReturnExchangeOrderProductInfo::type_one){
                $order['product_one'][] = $value;
            }else{
                $order['product_two'][] = $value;
            }
        }

        $order['crt_name'] = Employee::where('id',$order['crt_id'])->value('emp_name');
        $order['crt_time'] = $order['crt_time'] ? date("Y-m-d H:i:s",$order['crt_time']): '';
        //可见范围
        $return = (new RangeService())->RangeDetail($data['id'],SeeRange::type_six);
        $order['depart'] = $return[0] ?? [];
        $order['employee'] = $return[1] ?? [];

        return [true, $order];
    }

    public function del($data,$user){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];
        $order = ReturnExchangeOrder::where('del_time',0)->where('id',$data['id'])->first();
        if(empty($order)) return [false,'退换货单不存在或已被删除'];
        $order = $order->toArray();
        if($order['state'] > ReturnExchangeOrder::State_zero) return [false,'请确认退换货单状态,删除失败！'];

        $product_save_minus = $this->getSaveDetailMinus($data['id']);
        try{
            DB::beginTransaction();
            ReturnExchangeOrder::where('id',$data['id'])->update([
                'del_time'=>time()
            ]);
            ReturnExchangeOrderProductInfo::where('return_exchange_id',$data['id'])->update([
                'del_time'=>time()
            ]);
            //锁定库存
            ProductInventoryService::changeLockNumber($user, [], $product_save_minus);

            (new RangeService())->RangeDelete($data['id'],SeeRange::type_six);

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    public function getList($data,$user){
        $model = ReturnExchangeOrder::Clear($user,$data);
        $model = $model->where('del_time',0)
            ->select('id','order_number','data_id','difference_amount','state','crt_time','crt_id','mark','model_type','storehouse_id','type')
            ->orderby('id', 'desc');

        if(! empty($data['order_number'])) $model->where('order_number', 'LIKE', '%'.$data['order_number'].'%');
        if(isset($data['state'])) $model->where('state',$data['state']);
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->whereBetween('crt_time',[$return[0],$return[1]]);
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillListData($list);

        return [true, $list];
    }

    public function fillListData($data){
        if(empty($data['data'])) return $data;

        $emp_id = array_unique(array_merge_recursive(array_column($data['data'],'crt_id')));
        $emp_map = Employee::whereIn('id',$emp_id)
                    ->pluck('emp_name','id')
                    ->toArray();
        $sales = $purchase = [];
        foreach ($data['data'] as $value){
            if($value['type'] == ReturnExchangeOrder::Order_type){
                $sales[] = $value['data_id'];
            }elseif($value['type'] == ReturnExchangeOrder::Order_type2){
                $purchase[] = $value['data_id'];
            }
        }
        $sales = SalesOrder::whereIn('id',$sales)->pluck('order_number','id')->toArray();
        $purchase = PurchaseOrder::whereIn('id',$purchase)->pluck('order_number','id')->toArray();
        $storehouse = Storehouse::whereIn('id',array_unique(array_column($data['data'],'storehouse_id')))
            ->pluck('title','id')
            ->toArray();
        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date("Y-m-d H:i:s",$value['crt_time']) : '';
            $data['data'][$key]['crt_name'] = $emp_map[$value['crt_id']] ?? '';
            $data['data'][$key]['state_title'] = ReturnExchangeOrder::$state[$value['state']] ?? '';
            $title = "" ;
            if($value['type'] == ReturnExchangeOrder::Order_type){
                $title = $sales[$value['data_id']] ?? "";
            }elseif($value['type'] == ReturnExchangeOrder::Order_type2){
                $title = $purchase[$value['data_id']] ?? "";
            }
            $data['data'][$key]['data_title'] = $title;
            $data['data'][$key]['type_title'] = ReturnExchangeOrder::$order_type_name[$value['type']] ?? "";
            $data['data'][$key]['model_type_title'] = ReturnExchangeOrder::$model_type_name[$value['model_type']] ?? "";
            $data['data'][$key]['storehouse_title'] = $storehouse[$value['storehouse_id']] ?? '';
        }

        return $data;
    }

    public function orderRule(&$data, $user, $is_check = true){
        if($this->isEmpty($data,'model_type')) return [false,'单据类型不能为空！'];
        if(! in_array($data['model_type'], ReturnExchangeOrder::$model_type)) return [false,'单据类型不存在'];
        if(empty($data['storehouse_id'])) return [false,'请选择仓库'];
        if(! isset($data['type'])) return [false,'退换单据类型不能为空'];
        if(empty($data['data_id'])) return [false,'退换单据数据不能为空！'];
        if($data['type'] == ReturnExchangeOrder::Order_type){
            $sale = SalesOrder::where('del_time',0)->where('id',$data['data_id'])->first();
            if(empty($sale)) return [false,'合同不存在或已被删除'];
            if($sale['sales_order_type'] != SalesOrder::Order_type_one) return [false, '非安装件合同'];
            if($sale['state'] < SalesOrder::State_two) return [false,'合同未施工，不能进行退换货操作'];
        }elseif ($data['type'] == ReturnExchangeOrder::Order_type2){
            $purchase = PurchaseOrder::where('del_time',0)->where('id',$data['data_id'])->first();
            if(empty($purchase)) return [false,'采购单不存在或已被删除'];
            if($purchase['state'] < PurchaseOrder::STATE_ONE) return [false,'采购单未确认，不能进行退换货操作'];
        }

        //所属部门 以及  顶级部门
        if(empty($data['depart_id'])) {
            $data['depart_id'] = $this->getDepart($user);
            $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;
        }

        //退货
        $return_submit_add = [];
        if(empty($data['product_one'])) return [false,'退货产品不能为空'];
        foreach ($data['product_one'] as $value){
            if(empty($value['number'])) return [false,'退货产品数量不能为空'];
            $res = $this->checkNumber($value['number']);
            if(! $res) return [false,'请输入正确的退货产品数量'];
            if(isset($return_submit_add[$value['product_id']])){
                $return_submit_add[$value['product_id']] += $value['number'];
            }else{
                $return_submit_add[$value['product_id']] = $value['number'];
            }
        }

        if($data['type'] == ReturnExchangeOrder::Order_type){
            //剩余能退
            $id = $data['id'] ?? 0;
            $s_product = $this->getSaveReturnCompareMessage($id, $data['data_id']);

            //比较
            foreach ($return_submit_add as $pro => $number){
                if(! isset($s_product[$pro])) return [false,'退货产品错误,合同中不存在该产品'];
                $s_number = $s_product[$pro];

                if($number > $s_number) return [false,'退货产品数量不能超过合同产品数据'];
            }
        }elseif ($data['type'] == ReturnExchangeOrder::Order_type2){
            //剩余能退
            $id = $data['id'] ?? 0;
            $s_product = $this->getSaveReturnCompareMessage2($id, $data['data_id']);

            //比较
            foreach ($return_submit_add as $pro => $number){
                if(! isset($s_product[$pro])) return [false,'退货产品错误,采购单中不存在该产品'];
                $s_number = $s_product[$pro];

                if($number > $s_number) return [false,'退货产品数量不能超过合采购单产品数据'];
            }
        }

        //换货
        $product_submit_minus = $product_save_minus = $product_id = [];
        if($data['model_type'] == ReturnExchangeOrder::Model_type_two){
            if(empty($data['product_two'])) return [false,'换货产品不能为空'];
            foreach ($data['product_two'] as $value){
                if(empty($value['number'])) return [false,'换货产品数量不能为空'];
                $res = $this->checkNumber($value['number']);
                if(! $res) return [false,'请输入正确的换货产品数量'];

                $key = $value['product_id'] . ',' .$data['storehouse_id'];
                if(isset($product_submit_minus[$key])){
                    $product_submit_minus[$key] += $value['number'];
                }else{
                    $product_submit_minus[$key] = $value['number'];
                }
                $product_id[] = $value['product_id'];
            }

            //是否校验库存
            ProductInventoryService::is_check($user,$data);

            $product_save_minus = $this->getSaveDetailMinus($id);
            list($status,$msg) = (new ProductInventoryService())->compareStock($user,$product_id, $product_submit_minus, $product_save_minus);
            if(! $status) return [false, $msg];
        }

        if($is_check){
            $prefix = ReturnExchangeOrder::$prefix[$data['model_type']] ?? "";
            $order_number = (new OrderNoService())->createReturnExchangeOrderNumber($prefix);
            if(empty($order_number)) return [false,'退换货单号生成失败！'];
            $data['order_number'] = $order_number;
        }else{
            if($this->isEmpty($data,'id')) return [false,'退换货单ID不能为空！'];
            if($this->isEmpty($data,'order_number')) return [false,'退换货单号不能为空！'];
        }

        return [true, [$product_submit_minus,$product_save_minus]];
    }

    /**
     * 获取保存详情
     * @param $order_number
     * @return array
     */
    public function getSaveDetailMinus($id = 0){
        $product_save = [];
        if(empty($id)) return $product_save;

        $sub = ReturnExchangeOrderProductInfo::where('return_exchange_id',$id)
            ->where('return_or_exchange',ReturnExchangeOrderProductInfo::type_two)
            ->where('del_time',0)
            ->get()->toArray();
        foreach ($sub as $value){
            $key = $value['product_id'] . ',' . $value['storehouse_id'];
            if(isset($product_save[$key])){
                $product_save[$key] += $value['number'];
            }else{
                $product_save[$key] = $value['number'];
            }
        }

        return $product_save;
    }

    //合同剩余能退
    public function getSaveReturnCompareMessage($id = 0, $sales_order_id = 0){
        $return = ReturnExchangeOrder::where('del_time',0)
            ->where('data_id',$sales_order_id)
            ->select('id')->get()->toArray();
        $return = array_column($return,'id');
        $product_save = [];
        $sub = ReturnExchangeOrderProductInfo::where('return_or_exchange',ReturnExchangeOrderProductInfo::type_one)
            ->where('del_time',0)
            ->whereIn('return_exchange_id',$return)
            ->when(! empty($id), function ($query) use ($id) {
                return $query->where('return_exchange_id', '<>',$id);
            })
            ->get()->toArray();
        foreach ($sub as $value){
            if(isset($product_save[$value['product_id']])){
                $product_save[$value['product_id']] += $value['number'];
            }else{
                $product_save[$value['product_id']] = $value['number'];
            }
        }

        $sales_order_product = [];
        $sales_product = SalesOrderProductInfo::where('del_time',0)
            ->where('sales_order_id',$sales_order_id)
            ->get()->toArray();
        foreach ($sales_product as $value){
            $product_save_tmp = $product_save[$value['product_id']] ?? 0;
            if(isset($sales_order_product[$value['product_id']])){
                $sales_order_product[$value['product_id']] += $value['number'];
            }else{
                $sales_order_product[$value['product_id']] = $value['number'] - $product_save_tmp;
            }
        }

        return $sales_order_product;
    }

    //采购剩余能退
    public function getSaveReturnCompareMessage2($id = 0, $purchase_id = 0){
        $return = ReturnExchangeOrder::where('del_time',0)
            ->where('data_id',$purchase_id)
            ->select('id')->get()->toArray();
        $return = array_column($return,'id');
        $product_save = [];
        $sub = ReturnExchangeOrderProductInfo::where('return_or_exchange',ReturnExchangeOrderProductInfo::type_one)
            ->where('del_time',0)
            ->whereIn('return_exchange_id',$return)
            ->when(! empty($id), function ($query) use ($id) {
                return $query->where('return_exchange_id', '<>',$id);
            })
            ->get()->toArray();
        foreach ($sub as $value){
            if(isset($product_save[$value['product_id']])){
                $product_save[$value['product_id']] += $value['number'];
            }else{
                $product_save[$value['product_id']] = $value['number'];
            }
        }

        $sales_order_product = [];
        $sales_product = PurchaseOrderInfo::where('del_time',0)
            ->where('purchase_order_id',$purchase_id)
            ->get()->toArray();
        foreach ($sales_product as $value){
            $product_save_tmp = $product_save[$value['product_id']] ?? 0;
            if(isset($sales_order_product[$value['product_id']])){
                $sales_order_product[$value['product_id']] += $value['number'];
            }else{
                $sales_order_product[$value['product_id']] = $value['number'] - $product_save_tmp;
            }
        }

        return $sales_order_product;
    }

    //获取差异金额
    public function getDifferentAmount($id = 0, $type = 0){
        if(empty($id)) return [];

        $result = ReturnExchangeOrder::where('del_time',0)
            ->where('data_id',$id)
            ->where('type',$type)
            ->select('id','model_type','order_number','difference_amount','state')
            ->get()->toArray();
        foreach ($result as $key => $value){
            $result[$key]['model_type_title'] = ReturnExchangeOrder::$model_type[$value['model_type']] ?? "";
            $result[$key]['state_title'] = ReturnExchangeOrder::$state[$value['state']] ?? "";
        }

        return $result;
    }
}
