<?php

namespace App\Service;

use App\Model\BasicType;
use App\Model\BookingList;
use App\Model\Construction;
use App\Model\Customer;
use App\Model\Employee;
use App\Model\SalesOrder;
use App\Model\SalesOrderInfo;
use App\Model\SalesOrderOtherFee;
use App\Model\SalesOrderProductInfo;
use App\Model\SeeRange;
use App\Model\SportsBag;
use App\Model\SportsBagProductInfo;
use Illuminate\Support\Facades\DB;

/**
 * 订单管理
 */
class SalesOrderService extends Service
{
    /**
     * 订单编辑
     * @param $data
     * @param $user
     * @return array
     */
    public function salesOrderEdit($data,$user){
        list($status,$msg) = $this->salesOrderRule($data, $user, false);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = SalesOrder::where('id', $data['id'])->first();
            $model->sales_order_type = $data['sales_order_type'];
            $model->model_type = $data['model_type'];
            $model->order_number = $data['order_number'];
            $model->title = $data['title'] ?? '';
            $model->selling_price = $data['selling_price'] ?? 0;
            $model->vin_no = $data['vin_no'] ?? '';
            $model->order_type = $data['order_type'] ?? 0;
            $model->deal_type = $data['deal_type'] ?? 0;
            $model->customer_id = $data['customer_id'] ?? 0;
            $model->sign_time = $data['sign_time'] ?? 0;
            $model->contract_state = $data['contract_state'] ?? 0;
            $model->product_total = $data['product_total'] ?? 0;
            $model->rate = $data['rate'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->construction_time = $data['construction_time'] ?? 0;
            $model->handover_time = $data['handover_time'] ?? 0;
            $model->expire_time = $data['expire_time'] ?? 0;
            $model->other_fee = $data['other_fee'] ?? 0;
            $model->discount_fee = $data['discount_fee'] ?? 0;
            $model->contract_fee = $data['contract_fee'] ?? 0;
            $model->contract_type = $data['contract_type'] ?? 0;
            $model->pay_way = $data['pay_way'] ?? 0;
            $model->send_state = $data['send_state'] ?? 0;
            $model->logistics_company = $data['logistics_company'] ?? 0;
            $model->logistics_number = $data['logistics_number'] ?? '';
            $model->car_type = $data['car_type'] ?? 0;
            $model->year = $data['year'] ?? '';
            $model->mileage = $data['mileage'] ?? '';
            $model->color = $data['color'] ?? '';
            $model->original_set = $data['original_set'] ?? '';
            $model->processing = $data['processing'] ?? '';
            $model->save();
            $time = time();

            SalesOrderInfo::where('del_time',0)
                ->where('sales_order_id',$data['id'])
                ->update(['del_time' => $time]);
            SalesOrderProductInfo::where('del_time',0)
                ->where('sales_order_id',$data['id'])
                ->update(['del_time' => $time]);

            if(! empty($data['employee_one'])){
                $insert = [];
                foreach ($data['employee_one'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'data_id' => $value,
                        'type' => SalesOrderInfo::type_one,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['employee_two'])){
                $insert = [];
                foreach ($data['employee_two'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'data_id' => $value,
                        'type' => SalesOrderInfo::type_two,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['employee_three'])){
                $insert = [];
                foreach ($data['employee_three'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'data_id' => $value,
                        'type' => SalesOrderInfo::type_three,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['file'])){
                $insert = [];
                foreach ($data['file'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'file' => $value['url'],
                        'name' => $value['name'],
                        'type' => SalesOrderInfo::type_four,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['product'])){
                $insert = [];
                foreach ($data['product'] as $value){
                    $sports_bag_product_info_id = 0;
                    if(! empty($value['sports_bag_id'])) $sports_bag_product_info_id = $value['id'];
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'cost' => $value['cost'] ?? 0,
                        'retail_price' => $value['retail_price'] ?? 0,
                        'mark' => $value['mark'] ?? '',
                        'crt_time' => $time,
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'final_amount' => $value['final_amount'] ?? 0,
                        'sports_bag_id' => $value['sports_bag_id'] ?? 0,
                        'sports_bag_product_info_id' => $sports_bag_product_info_id,
                    ];
                }
                SalesOrderProductInfo::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    /**
     * 产品新增
     * @param $data
     * @param $user
     * @return array
     */
    public function salesOrderAdd($data,$user){
        list($status,$msg) = $this->salesOrderRule($data,$user);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = new SalesOrder();
            $model->sales_order_type = $data['sales_order_type'];
            $model->model_type = $data['model_type'];
            $model->order_number = $data['order_number'];
            $model->title = $data['title'] ?? '';
            $model->selling_price = $data['selling_price'] ?? 0;
            $model->vin_no = $data['vin_no'] ?? '';
            $model->order_type = $data['order_type'] ?? 0;
            $model->deal_type = $data['deal_type'] ?? 0;
            $model->customer_id = $data['customer_id'] ?? 0;
            $model->sign_time = $data['sign_time'] ?? 0;
            $model->contract_state = $data['contract_state'] ?? 0;
            $model->product_total = $data['product_total'] ?? 0;
            $model->rate = $data['rate'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->construction_time = $data['construction_time'] ?? 0;
            $model->handover_time = $data['handover_time'] ?? 0;
            $model->expire_time = $data['expire_time'] ?? 0;
            $model->other_fee = $data['other_fee'] ?? 0;
            $model->discount_fee = $data['discount_fee'] ?? 0;
            $model->contract_fee = $data['contract_fee'] ?? 0;
            $model->contract_type = $data['contract_type'] ?? 0;
            $model->pay_way = $data['pay_way'] ?? 0;
            $model->send_state = $data['send_state'] ?? 0;
            $model->logistics_company = $data['logistics_company'] ?? 0;
            $model->logistics_number = $data['logistics_number'] ?? '';
            $model->car_type = $data['car_type'] ?? 0;
            $model->year = $data['year'] ?? '';
            $model->mileage = $data['mileage'] ?? '';
            $model->color = $data['color'] ?? '';
            $model->original_set = $data['original_set'] ?? '';
            $model->processing = $data['processing'] ?? '';
            $model->depart_id = $data['depart_id'] ?? 0;
            $model->top_depart_id = $data['top_depart_id'] ?? 0;
            $model->crt_id = $user['id'];
            $model->save();
            $time = time();

            if(! empty($data['employee_one'])){
                $insert = [];
                foreach ($data['employee_one'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'data_id' => $value,
                        'type' => SalesOrderInfo::type_one,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['employee_two'])){
                $insert = [];
                foreach ($data['employee_two'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'data_id' => $value,
                        'type' => SalesOrderInfo::type_two,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['employee_three'])){
                $insert = [];
                foreach ($data['employee_three'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'data_id' => $value,
                        'type' => SalesOrderInfo::type_three,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['file'])){
                $insert = [];
                foreach ($data['file'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'file' => $value['url'],
                        'name' => $value['name'],
                        'type' => SalesOrderInfo::type_four,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['product'])){
                $insert = [];
                foreach ($data['product'] as $value){
                    $sports_bag_product_info_id = 0;
                    if(! empty($value['sports_bag_id'])) $sports_bag_product_info_id = $value['id'];
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'cost' => $value['cost'] ?? 0,
                        'retail_price' => $value['retail_price'] ?? 0,
                        'mark' => $value['mark'] ?? '',
                        'crt_time' => $time,
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'final_amount' => $value['final_amount'] ?? 0,
                        'sports_bag_id' => $value['sports_bag_id'] ?? 0,
                        'sports_bag_product_info_id' => $sports_bag_product_info_id,
                    ];
                }
                SalesOrderProductInfo::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getLine().':'.$exception->getMessage()];
        }

        return [true,''];
    }

    /**
     * 订单删除
     * @param $data
     * @return array
     */
    public function salesOrderDel($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];
        $sale_order = SalesOrder::where('id',$data['id'])->first();
        if(empty($sale_order)) return [false,'合同不存在或已被删除'];
        $sale_order = $sale_order->toArray();
        if($sale_order['state'] > 0) return [false,'请确认合同状态,删除失败'];

        try {
            DB::beginTransaction();

            SalesOrder::where('del_time',0)
                ->where('id',$data['id'])->update(['del_time'=> time()]);
            SalesOrderInfo::where('del_time',0)
                ->where('sales_order_id',$data['id'])
                ->update(['del_time' => time()]);
            SalesOrderProductInfo::where('del_time',0)
                ->where('sales_order_id',$data['id'])
                ->update(['del_time' => time()]);
            SalesOrderOtherFee::where('del_time',0)
                ->where('sales_order_id',$data['id'])
                ->update(['del_time' => time()]);
            (new RangeService())->RangeDelete($data['id'],SeeRange::type_seven);

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    /**
     * 订单详情
     * @param $data
     * @return array
     */
    public function salesOrderDetail($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];

        $sales = SalesOrder::where('del_time',0)
            ->where('id',$data['id'])
            ->first();
        if(empty($sales)) return [false,'合同不存在或已被删除'];
        $sales = $sales->toArray();
        $customer = Customer::where('id',$sales['customer_id'])->value('title');
        $sales['customer_title'] = $customer ?? '';
        $sales['sales_order_type_title'] = SalesOrder::$order_type[$sales['sales_order_type']] ?? '';
        $sales['activity_product'] = $sales['file'] = $sales['employee_one'] = $sales['employee_two'] = $sales['employee_three'] = $sales['product'] = $sales['dispatch_depart_one'] = $sales['dispatch_employee'] = $sales['dispatch_depart_two'] = [];
        $array = [
            $sales['order_type'],
            $sales['deal_type'],
            $sales['contract_state'],
            $sales['contract_type'],
            $sales['car_type'],
            $sales['pay_way'],
            $sales['send_state'],
            $sales['logistics_company'],
        ];
        $basic_map = BasicType::whereIn('id',$array)
            ->pluck('title','id')
            ->toArray();
        $sales = [$sales];
        foreach ($sales as $key => $value){
            $sales[$key]['order_type_title'] = $basic_map[$value['order_type']] ?? '';
            $sales[$key]['deal_type_title'] = $basic_map[$value['deal_type']] ?? '';
            $sales[$key]['contract_state_title'] = $basic_map[$value['contract_state']] ?? '';
            $sales[$key]['contract_type_title'] = $basic_map[$value['contract_type']] ?? '';
            $sales[$key]['car_type_title'] = $basic_map[$value['car_type']] ?? '';
            $sales[$key]['pay_way_title'] = $basic_map[$value['pay_way']] ?? '';
            $sales[$key]['send_state_title'] = $basic_map[$value['send_state']] ?? '';
            $sales[$key]['logistics_company_title'] = $basic_map[$value['logistics_company']] ?? '';
        }
        $sales = $sales[0];

        $sales_info = SalesOrderInfo::where('del_time',0)
            ->where('sales_order_id',$sales['id'])
            ->get()->toArray();
        $emp_id = [];
        $emp_id[] = $sales['crt_id'];
        foreach ($sales_info as $value){
            if(in_array($value['type'],SalesOrderInfo::$man)){
                $emp_id[] = $value['data_id'];
            }
        }

        $emp_map = Employee::whereIn('id',array_unique($emp_id))
            ->pluck('emp_name','id')
            ->toArray();
        foreach ($sales_info as $value){
            if(in_array($value['type'],SalesOrderInfo::$man)){
                $tmp = [
                    'id' => $value['data_id'],
                    'name' => $emp_map[$value['data_id']] ?? '',
                ];
                if($value['type'] == SalesOrderInfo::type_one){
                    $sales['employee_one'][] = $tmp;
                }elseif ($value['type'] == SalesOrderInfo::type_two){
                    $sales['employee_two'][] = $tmp;
                }elseif ($value['type'] == SalesOrderInfo::type_three){
                    $sales['employee_three'][] = $tmp;
                }
            } elseif ($value['type'] == SalesOrderInfo::type_four){
                $sales['file'][] = [
                    'url' => $value['file'],
                    'name' => $value['name'],
                ];
            }
        }

        $sales_p_info = SalesOrderProductInfo::where('del_time',0)
            ->where('sales_order_id',$sales['id'])
            ->get()->toArray();
        $bag = SportsBag::whereIn('id',array_unique(array_column($sales_p_info,'sports_bag_id')))->get()->toArray();
        $bag_pro = SportsBagProductInfo::where('del_time',0)->whereIn('sports_bag_id',array_unique(array_column($sales_p_info,'sports_bag_id')))
            ->get()->toArray();
        $bag_map = array_column($bag,null,'id');
        $bag_pro_map = [];
        foreach ($bag_pro as $value){
            $bag_pro_map[$value['sports_bag_id']][] = $value;
        }

        $basic_price = BasicType::whereIn('id',array_unique(array_column($sales_p_info,'basic_type_id')))->pluck('title','id')->toArray();
        $map = (new ProductService())->getProductDetail(array_column($sales_p_info,'product_id'));
        foreach ($sales_p_info as $value){
            $tmp = $map[$value['product_id']] ?? [];
            if(!empty($value['sports_bag_id'])){
                $bag_tmp = $bag_map[$value['sports_bag_id']] ?? [];
                $bag_pro_tmp = $bag_pro_map[$value['sports_bag_id']] ?? [];
                foreach ($bag_pro_tmp as $k => $v){
                    $is_choose = false;
                    $final_amount = 0;
                    $number = $v['number'];
                    if($v['id'] == $value['sports_bag_product_info_id']){
                        $is_choose = true;
                        $final_amount = $value['final_amount'];
                        $number = $value['number'];
                    }
                    $bag_pro_tmp[$k]['is_choose'] = $is_choose;
                    $bag_pro_tmp[$k]['final_amount'] = $final_amount;
                    $bag_pro_tmp[$k]['number'] = $number;
                    $bag_pro_tmp[$k]['title'] = $tmp['title'] ?? "";
                    $bag_pro_tmp[$k]['code'] = $tmp['code'] ?? "";
                    $bag_pro_tmp[$k]['size'] = $tmp['size'] ?? "";
                    $bag_pro_tmp[$k]['unit'] = $tmp['unit'] ?? "";
                    $bag_pro_tmp[$k]['bar_code'] = $tmp['bar_code'] ?? "";
                }
                $bag_tmp['product'] = $bag_pro_tmp;
                $sales['activity_product'][] = $bag_tmp;
            }else{
                $value['title'] = $tmp['title'] ?? "";
                $value['code'] = $tmp['code'] ?? "";
                $value['size'] = $tmp['size'] ?? "";
                $value['unit'] = $tmp['unit'] ?? "";
                $value['bar_code'] = $tmp['bar_code'] ?? "";
                $value['basic_type_title'] = $basic_price[$value['basic_type_id']] ?? "";
                $sales['product'][] = $value;
            }
        }
        $sales['crt_name'] = $emp_map[$sales['crt_id']] ?? '';
        $sales['crt_time'] = $sales['crt_time'] ? date("Y-m-d H:i:s",$sales['crt_time']): '';

        //派单后数据 也就是可见范围
        $return = (new RangeService())->RangeDetail($data['id'],SeeRange::type_seven);
        $sales['dispatch_depart_one'] = $return[0] ?? [];
        $sales['dispatch_employee'] = $return[1] ?? [];
        $sales['dispatch_depart_two'] = $return[2] ?? [];

        //退换货信息
        $sales['return_exchange'] = (new ReturnExchangeOrderService())->getDifferentAmount($data['id']);

        return [true, $sales];
    }

    /**
     * 订单列表
     * @param $data
     * @param $user
     * @return array
     */
    public function salesOrderList($data,$user){
        $model = SalesOrder::Clear($user,$data);
        $model = $model->where('del_time',0)
            ->select('title','id','sales_order_type','model_type','order_number','selling_price','vin_no','car_type','order_type','deal_type','customer_id','sign_time','contract_state','crt_id','crt_time','mark','product_total','rate','construction_time','handover_time','expire_time','other_fee','discount_fee','contract_fee','contract_type','pay_way','send_state','logistics_company','logistics_number','car_type','year','mileage','color','original_set','processing','state')
            ->orderby('id', 'desc');

        if(! empty($data['sales_order_type'])) $model->where('sales_order_type',$data['sales_order_type']);
        if(! empty($data['title'])) $model->where('title', 'LIKE', '%'.$data['title'].'%');
        if(! empty($data['model_type'])) $model->where('model_type',$data['model_type']);
        if(! empty($data['time_type'])) {
            if($data['time_type'] == 1) {
                $start = strtotime('today');
                $end = strtotime('tomorrow') - 1;
            }elseif ($data['time_type'] == 2){
                $start = strtotime('this week',strtotime('today'));
                $end = strtotime('this week +6 days 23:59:59', strtotime('today'));
            }
            if(! empty($start) && ! empty($end)) {
                $model->where('crt_time','>=',$start);
                $model->where('crt_time','<=',$end);
            }
        }
        if(! empty($data['add_construction'])) $model->where('state','>=',SalesOrder::State_two);
        if(! empty($data['add_return'])) $model->where('state','>',SalesOrder::State_two);
        if(! empty($data['sales_order_id'])) $model->where('id',$data['sales_order_id']);

        $list = $this->limit($model,'',$data);
        $list = $this->fillData($list,$data);

        return [true, $list];
    }

    /**
     * 订单参数规则
     * @param $data
     * @param $user
     * @param $is_add
     * @return array
     */
    public function salesOrderRule(&$data, $user, $is_add = true){
        if(empty($data['sales_order_type'])) return [false,'合同类型不能为空'];
        if(empty($data['model_type'])) return [false,'订单模板类型不能为空'];
        if(! in_array($data['model_type'],SalesOrder::$model_type)) return [false,'订单模板类型错误'];
        if(empty($data['order_number'])) return [false,'合同编号不能为空'];
//        if(empty($data['customer_id'])) return [false,'请选择客户'];
//        if(empty($data['employee_one'])) return [false,'请选择签订人'];
        if(! empty($data['sign_time'])) $data['sign_time'] = $this->changeDateToDateMin($data['sign_time']);
        if(! empty($data['expire_time'])) $data['expire_time'] = $this->changeDateToDateMin($data['expire_time']);
//        if(empty($data['contract_state'])) return [false,'请选择合同状态'];
//        if(empty($data['employee_two'])) return [false,'请选择负责人'];
//        if(empty($data['product'])) return [false,'请选择产品'];
        if(! empty($data['product'])){
            foreach ($data['product'] as $value){
                if(empty($value['number'])) return [false,'产品数量不能为空'];
                $res = $this->checkNumber($value['number']);
                if(! $res) return [false,'请输入正确的产品数量'];
            }
        }

        if(! empty($data['contract_fee'])){
            $res = $this->checkNumber($data['contract_fee']);
            if(! $res) return [false, '合同金额请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['other_fee'])){
            $res = $this->checkNumber($data['other_fee']);
            if(! $res) return [false, '其它费用请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['discount_fee'])){
            $res = $this->checkNumber($data['discount_fee']);
            if(! $res) return [false, '优惠金额请输入不超过两位小数并且大于0的数值'];
        }
        //所属部门 以及  顶级部门
        if(empty($data['depart_id'])) $data['depart_id'] = $this->getDepart($user);
        $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;

        if($data['model_type'] == SalesOrder::Model_type_one){
//            if(empty($data['order_type'])) return [false,'订单类型不能为空'];
            if(! empty($data['construction_time'])) $data['construction_time'] = $this->changeDateToDateMin($data['construction_time']);
            if(! empty($data['handover_time'])) $data['handover_time'] = $this->changeDateToDateMin($data['handover_time']);
//            if(empty($data['deal_type'])) return [false,'成交类型不能为空'];
        }elseif($data['model_type'] == SalesOrder::Model_type_two){
            if(empty($data['title'])) return [false,'合同名称不能为空'];
        }else{
//            if(empty($data['car_type'])) return [false,'车型不能为空'];
//            if(empty($data['year'])) return [false,'年份不能为空'];
//            if(empty($data['mileage'])) return [false,'表显里程不能为空'];
        }

        if($is_add){
            $bool = SalesOrder::where('del_time',0)->where('order_number',$data['order_number'])->exists();
            if($bool) return [false,'合同编号已存在，请重新获取'];

//            if($data['model_type'] == SalesOrder::Model_type_two){
//                $boolean = SalesOrder::where('del_time',0)
//                    ->where('title',$data['title'])
//                    ->where('model_type',$data['model_type'])
//                    ->exists();
//            }
        }else{
            if(empty($data['id'])) return [false,'ID不能为空'];
            $order = SalesOrder::where('del_time',0)->where('id',$data['id'])->first();
            if(empty($order)) return [false,'合同不存在或已被删除'];
            $order = $order->toArray();
            if($order != SalesOrder::State2_zero) return [false, '请确认合同状态，编辑失败'];

//            if($data['model_type'] == SalesOrder::Model_type_two){
//                $boolean = SalesOrder::where('del_time',0)
//                    ->where('id','<>',$data['id'])
//                    ->where('title',$data['title'])
//                    ->where('model_type',$data['model_type'])
//                    ->exists();
//            }
        }
//        if(isset($boolean) && $boolean) return [false,'合同名称不能重复'];

        return [true, $data];
    }

    /**
     * 数据拼接
     * @param $data
     * @param $ergs
     * @return array
     */
    public function fillData($data,$ergs){
        if(empty($data['data'])) return $data;

        $array = array_unique(array_merge_recursive(array_column($data['data'],'order_type'),array_column($data['data'],'deal_type'),array_column($data['data'],'contract_state'),array_column($data['data'],'car_type'),array_column($data['data'],'pay_way'),array_column($data['data'],'logistics_company'),array_column($data['data'],'send_state'),array_column($data['data'],'contract_type')));
        $basic_map = BasicType::whereIn('id',$array)
            ->pluck('title','id')
            ->toArray();
        $emp = Employee::whereIn('id',array_unique(array_column($data['data'],'crt_id')))
            ->pluck('emp_name','id')
            ->toArray();
        $customer = Customer::whereIn('id',array_unique(array_column($data['data'],'customer_id')))
            ->pluck('title','id')
            ->toArray();

        $product = $fee = [];
        if(! empty($ergs['product'])){
            $sales_p_info = SalesOrderProductInfo::where('del_time',0)
                ->whereIn('sales_order_id',array_column($data['data'],'id'))
                ->get()->toArray();
            $basic_price = BasicType::whereIn('id',array_unique(array_column($sales_p_info,'basic_type_id')))->pluck('title','id')->toArray();
            $map = (new ProductService())->getProductDetail(array_column($sales_p_info,'product_id'));

            foreach ($sales_p_info as $value){
                $tmp = $map[$value['product_id']] ?? [];
                $value['title'] = $tmp['title'] ?? "";
                $value['code'] = $tmp['code'] ?? "";
                $value['size'] = $tmp['size'] ?? "";
                $value['unit'] = $tmp['unit'] ?? "";
                $value['bar_code'] = $tmp['bar_code'] ?? "";
                $value['basic_type_title'] = $basic_price[$value['basic_type_id']] ?? "";
                $product[$value['sales_order_id']][] = $value;
            }
            $sales_o_info = SalesOrderOtherFee::where('del_time',0)
                ->where('sales_order_id',array_column($data['data'],'id'))
                ->get()->toArray();
            foreach ($sales_o_info as $value){
                $fee[$value['sales_order_id']][] = $value;
            }
        }

        //获取金额列表
        $money = (new BookingListService())->getAllAmount(array_column($data['data'],'id'), BookingList::data_type_one);
        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['sales_order_type_title'] = SalesOrder::$order_type[$value['sales_order_type']] ?? '';
            $data['data'][$key]['order_type_title'] = $basic_map[$value['order_type']] ?? '';
            $data['data'][$key]['deal_type_title'] = $basic_map[$value['deal_type']] ?? '';
            $data['data'][$key]['contract_state_title'] = $basic_map[$value['contract_state']] ?? '';
            $data['data'][$key]['contract_type_title'] = $basic_map[$value['contract_type']] ?? '';
            $data['data'][$key]['car_type_title'] = $basic_map[$value['car_type']] ?? '';
            $data['data'][$key]['pay_way_title'] = $basic_map[$value['pay_way']] ?? '';
            $data['data'][$key]['send_state_title'] = $basic_map[$value['send_state']] ?? '';
            $data['data'][$key]['logistics_company_title'] = $basic_map[$value['logistics_company']] ?? '';
            $data['data'][$key]['customer_title'] = $customer[$value['customer_id']] ?? '';
            $data['data'][$key]['sign_time'] = $value['sign_time'] ? date('Y-m-d H:i:s',$value['sign_time']) : '';
            $data['data'][$key]['construction_time'] = $value['construction_time'] ? date('Y-m-d H:i:s',$value['construction_time']) : '';
            $data['data'][$key]['handover_time'] = $value['handover_time'] ? date('Y-m-d H:i:s',$value['handover_time']) : '';
            $data['data'][$key]['expire_time'] = $value['expire_time'] ? date('Y-m-d H:i:s',$value['expire_time']) : '';
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date('Y-m-d H:i:s',$value['crt_time']) : '';
            $data['data'][$key]['crt_name'] = $emp[$value['crt_id']] ?? '';
            $data['data'][$key]['product'] = $product[$value['id']] ?? [];
            $data['data'][$key]['fee'] = $fee[$value['id']] ?? [];
            if($value['sales_order_type'] == SalesOrder::Order_type_one){
                $data['data'][$key]['state_name'] = SalesOrder::$state[$value['state']] ?? '';
            }else{
                $data['data'][$key]['state_name'] = SalesOrder::$state2[$value['state']] ?? '';
            }

            //合同对应的金额数据
            $tmp = $money[$value['id']] ?? [];
            $fee1 = $tmp[BookingList::type_one] ?? 0; //收款金额
            $fee2 = $tmp[BookingList::type_two] ?? 0; //坏账金额
            $fee3 = $tmp[BookingList::type_three] ?? 0; //退款金额
            $fee4 = $tmp[BookingList::type_four] ?? 0; //补收金额

            //已收金额
            $data['data'][$key]['fee1'] = $fee1 + $fee4;
            //未收金额
            $data['data'][$key]['fee2'] = $value['contract_fee'] - ($fee1 + $fee4);
            //坏账金额
            $data['data'][$key]['fee3'] = $fee2;
            //最终金额
            $data['data'][$key]['fee4'] = $value['contract_fee'] + $fee4 - $fee3;
        }

        return $data;
    }

    public function salesOrderGet($data){
        if(empty($data['model_type'])) return [false,'订单模板类型不能为空'];
        if(! isset(SalesOrder::$prefix[$data['model_type']])) return [false,'订单模板类型错误'];
        $prefix = SalesOrder::$prefix[$data['model_type']];
        $order_number = OrderNoService::createSalesOrderNumber($prefix);
        if(! $order_number) return [false,'合同编号生成失败！'];

        return [true, ['order_number' => $order_number]];
    }

    /**
     * 订单推送
     * @param $data
     * @return array
     */
    public function salesOrderDispatchSale($data){
        list($status,$msg) = $this->salesOrderDispatchSaleRule($data);
        if(! $status) return [false,$msg];

        try {
            DB::beginTransaction();
            SalesOrder::where('del_time',0)->where('id',$msg['id'])->update([
                'state' => SalesOrder::State_one
            ]);

            $time = time();
            SeeRange::where('del_time',0)
                ->where('data_id',$msg['id'])
                ->where('data_type',SeeRange::type_seven)
                ->whereIn('type',[SeeRange::data_one, SeeRange::data_two])
                ->update(['del_time' => $time]);

            if(! empty($data['dispatch_depart_one'])){
                $insert = [];
                foreach ($data['dispatch_depart_one'] as $value){
                    $insert[] = [
                        'data_id' => $msg['id'],
                        'data_type' => SeeRange::type_seven,
                        'param_id' => $value,
                        'type' => SeeRange::data_one,
                        'crt_time' => $time,
                    ];
                }
                SeeRange::insert($insert);
            }

            if(! empty($data['dispatch_employee'])){
                $insert = [];
                foreach ($data['dispatch_employee'] as $value){
                    $insert[] = [
                        'data_id' => $msg['id'],
                        'data_type' => SeeRange::type_seven,
                        'param_id' => $value,
                        'type' => SeeRange::data_two,
                        'crt_time' => $time,
                    ];
                }
                SeeRange::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    /**
     * 订单推送规则
     * @param $data
     * @return array
     */
    public function salesOrderDispatchSaleRule($data){
        if(empty($data['id'])) return [false,'请选择合同派单'];
        $sale = SalesOrder::where('del_time',0)
            ->where('id',$data['id'])
            ->first();
        if(empty($sale)) return [false,'合同不存在或已被删除'];
        $sale = $sale->toArray();
        if($sale['sales_order_type'] != SalesOrder::Order_type_one) return [false,'非安装件合同，操作失败'];
        if($sale['state'] > SalesOrder::State_one) return [false,'请确认合同状态，操作失败'];
        $bool = SalesOrderProductInfo::where('del_time',0)
            ->where('sales_order_id',$data['id'])
            ->exists();
        if(! $bool) return [false,'合同产品不能为空'];
        if(empty($data['dispatch_depart_one']) && empty($data['dispatch_employee'])) return [false, '请指派部门或人员'];

        return [true, $sale];
    }

    /**
     * 订单推送公司
     * @param $data
     * @return array
     */
    public function salesOrderDispatchCompany($data){
        list($status,$msg) = $this->salesOrderDispatchCompanyRule($data);
        if(! $status) return [false,$msg];

        try {
            DB::beginTransaction();
            SalesOrder::where('del_time',0)->where('id',$msg['id'])->update([
                'state' => SalesOrder::State_two
            ]);

            $time = time();
            SeeRange::where('del_time',0)
                ->where('data_id',$msg['id'])
                ->where('data_type',SeeRange::type_seven)
                ->where('type',SeeRange::data_three)
                ->update(['del_time' => $time]);
            SalesOrderOtherFee::where('del_time',0)
                ->where('sales_order_id',$msg['id'])
                ->update(['del_time' => $time]);
            if(! empty($data['dispatch_depart_two'])){
                $insert = [];
                foreach ($data['dispatch_depart_two'] as $value){
                    $insert[] = [
                        'data_id' => $msg['id'],
                        'data_type' => SeeRange::type_seven,
                        'param_id' => $value,
                        'type' => SeeRange::data_three,
                        'crt_time' => $time,
                    ];
                }
                SeeRange::insert($insert);
            }

            if(! empty($data['other_fee_1'])){
                $model = new SalesOrderOtherFee();
                $model->sales_order_id = $msg['id'];
                $model->other_fee_1 = $data['other_fee_1'] ?? 0;
                $model->save();
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    /**
     * 订单推送公司规则
     * @param $data
     * @return array
     */
    public function salesOrderDispatchCompanyRule($data){
        if(empty($data['id'])) return [false,'请选择合同派单'];
        $sale = SalesOrder::where('del_time',0)
            ->where('id',$data['id'])
            ->first();
        if(empty($sale)) return [false,'合同不存在或已被删除'];
        $sale = $sale->toArray();
        if($sale['sales_order_type'] != SalesOrder::Order_type_one) return [false,'非安装件合同，操作失败'];
        if($sale['state'] > SalesOrder::State_two) return [false,'请确认合同状态，操作失败'];
        $bool = SalesOrderProductInfo::where('del_time',0)
            ->where('sales_order_id',$data['id'])
            ->exists();
        if(! $bool) return [false,'合同产品不能为空'];

        $bool = Construction::where('del_time',0)
            ->where('sales_order_id',$data['id'])
            ->exists();
        if($bool) return [false,'合同已下施工，操作失败'];

        if(empty($data['dispatch_depart_two'])) return [false, '请指派总社或分社'];
        if(empty($data['other_fee_1'])) return [false,'价格不能为空'];
        $res = $this->checkNumber($data['other_fee_1']);
        if(! $res) return [false,'价格请输入不超过两位小数并且大于0的数值'];

        return [true, $sale];
    }
}
