<?php

namespace App\Service;

use App\Model\Employee;
use App\Model\SeeRange;
use App\Model\SportsBag;
use App\Model\SportsBagProductInfo;
use Illuminate\Support\Facades\DB;

class SportsBagService extends Service
{
    public function edit($data,$user){
        list($status,$msg) = $this->orderRule($data,$user,false);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $material_model = SportsBag::where('id',$data['id'])->first();
            $material_model->title = $data['title'];
            $material_model->mark = $data['mark'];
            $material_model->number = $data['number'];
            $material_model->start_time = $data['start_time'] ?? 0;
            $material_model->end_time = $data['end_time'] ?? 0;
            $material_model->total_amount = $data['total_amount'] ?? 0;
            $material_model->save();
            $time = time();

            SportsBagProductInfo::where('del_time',0)
                ->where('sports_bag_id',$data['id'])
                ->update(['del_time' => $time]);
            if(!empty($data['product'])){
                $map = (new ProductService())->getProductDetail(array_column($data['product'],'product_id'));
                $sub = [];
                foreach ($data['product'] as $value){
                    $tmp = $map[$value['product_id']] ?? [];
                    $sub[] = [
                        'sports_bag_id' => $material_model->id,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                        'cost' => $value['cost'] ?? 0,
                        'retail_price' => $tmp['retail_price'] ?? 0,
                        'crt_time' => $time,
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                    ];
                }
                SportsBagProductInfo::insert($sub);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    public function add($data,$user){
        list($status,$msg) = $this->orderRule($data,$user);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $material_model = new SportsBag();
            $material_model->title = $data['title'];
            $material_model->mark = $data['mark'];
            $material_model->number = $data['number'];
            $material_model->start_time = $data['start_time'] ?? 0;
            $material_model->end_time = $data['end_time'] ?? 0;
            $material_model->crt_id = $user['id'];
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->top_depart_id = $data['top_depart_id'] ?? 0;
            $material_model->total_amount = $data['total_amount'] ?? 0;
            $material_model->save();
            $time = time();

            if(!empty($data['product'])){
                $map = (new ProductService())->getProductDetail(array_column($data['product'],'product_id'));
                $sub = [];
                foreach ($data['product'] as $value){
                    $tmp = $map[$value['product_id']] ?? [];
                    $sub[] = [
                        'sports_bag_id' => $material_model->id,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                        'cost' => $value['cost'] ?? 0,
                        'retail_price' => $tmp['retail_price'] ?? 0,
                        'crt_time' => $time,
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                    ];
                }
                SportsBagProductInfo::insert($sub);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    public function detail($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据'];

        $order = SportsBag::where('id',$data['id'])
            ->where('del_time',0)
            ->first();
        if(empty($order)) return [false, '活动包不存在或已被删除'];
        $order = $order->toArray();
        $start_time = $order['start_time'] ? date("Y-m-d H:i",$order['start_time']) : '';
        $end_time = $order['end_time'] ? date("Y-m-d H:i",$order['end_time']) : '';
        $order['sports_bag_time'] = $start_time . '——' . $end_time;

        $order['product'] = $order['depart'] = $order['employee'] = [];
        $info = SportsBagProductInfo::where('del_time',0)
            ->where('sports_bag_id',$data['id'])
            ->get()->toArray();
        $map = (new ProductService())->getProductDetail(array_column($info,'product_id'));
        foreach ($info as $value){
            $tmp = $map[$value['product_id']] ?? [];
            $value['title'] = $tmp['title'] ?? "";
            $value['code'] = $tmp['code'] ?? "";
            $value['size'] = $tmp['size'] ?? "";
            $value['unit'] = $tmp['unit'] ?? "";
            $value['bar_code'] = $tmp['bar_code'] ?? "";
            $order['product'][] = $value;
        }

        $order['crt_name'] = Employee::where('id',$order['crt_id'])->value('emp_name');
        $order['crt_time'] = $order['crt_time'] ? date("Y-m-d H:i:s",$order['crt_time']): '';
        //可见范围
        $return = (new RangeService())->RangeDetail($order['id'],SeeRange::type_eight);
        $order['depart'] = $return[0] ?? [];
        $order['employee'] = $return[1] ?? [];

        return [true, $order];
    }

    public function del($data,$user){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];
        $order = SportsBag::where('del_time',0)->where('id',$data['id'])->first();
        if(empty($order)) return [false,'活动包不存在或已被删除'];

        try{
            DB::beginTransaction();
            SportsBag::where('id',$data['id'])->update([
                'del_time'=>time()
            ]);
            SportsBagProductInfo::where('sports_bag_id',$data['id'])->update([
                'del_time'=>time()
            ]);
            (new RangeService())->RangeDelete($data['id'],SeeRange::type_eight);

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    public function getList($data,$user){
        $model = SportsBag::Clear($user,$data);
        $model = $model->where('del_time',0)
            ->select('id','title','number','start_time','end_time','crt_time','crt_id','mark','total_amount')
            ->orderby('id', 'desc');

        if(isset($data['valid'])) {
            //获取有效时间内的活动包
            $time = time();
            $model->where('start_time','<=',$time);
            $model->where('end_time','>=',$time);
        }
        if(! empty($data['title'])) $model->where('title', 'LIKE', '%'.$data['title'].'%');
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->where('crt_time','>=',$return[0]);
            $model->where('crt_time','<=',$return[1]);
        }
        if(! empty($data['sports_bag_time'][0]) && ! empty($data['sports_bag_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['sports_bag_time']);
            $model->where('start_time','>=',$return[0]);
            $model->where('end_time','<=',$return[1]);
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillListData($list,$data);

        return [true, $list];
    }

    public function fillListData($data,$ergs){
        if(empty($data['data'])) return $data;

        $emp_id = array_unique(array_merge_recursive(array_column($data['data'],'crt_id')));
        $emp_map = Employee::whereIn('id',$emp_id)
                    ->pluck('emp_name','id')
                    ->toArray();
        $product = [];
        if(! empty($ergs['product'])){
            $sales_p_info = SportsBagProductInfo::where('del_time',0)
                ->where('sports_bag_id',array_column($data['data'],'id'))
                ->get()->toArray();
            $map = (new ProductService())->getProductDetail(array_column($sales_p_info,'product_id'));
            foreach ($sales_p_info as $value){
                $tmp = $map[$value['product_id']] ?? [];
                $value['title'] = $tmp['title'] ?? "";
                $value['code'] = $tmp['code'] ?? "";
                $value['size'] = $tmp['size'] ?? "";
                $value['unit'] = $tmp['unit'] ?? "";
                $value['bar_code'] = $tmp['bar_code'] ?? "";
                $product[$value['sports_bag_id']][] = $value;
            }
        }
        foreach ($data['data'] as $key => $value){
            $start_time = $value['start_time'] ? date("Y-m-d H:i",$value['start_time']) : '';
            $end_time = $value['end_time'] ? date("Y-m-d H:i",$value['end_time']) : '';
            $data['data'][$key]['sports_bag_time'] = $start_time . '——' . $end_time;
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date("Y-m-d H:i:s",$value['crt_time']) : '';
            $data['data'][$key]['crt_name'] = $emp_map[$value['crt_id']] ?? '';
            $data['data'][$key]['product'] = $product[$value['id']] ?? [];
        }

        return $data;
    }

    public function orderRule(&$data, $user, $is_check = true){
        if(empty($data['title'])) return [false,'请填写活动名称'];
        if(empty($data['sports_bag_time'][0]) || empty($data['sports_bag_time'][1])) return [false,'请填写活动时间范围'];
        $data['start_time'] = $this->changeDateToDateMin($data['sports_bag_time'][0]);
        $data['end_time'] = $this->changeDateToDateMin($data['sports_bag_time'][1]);
        if(empty($data['number'])) return [false,'必选产品种数不能为空'];
        $res = $this->checkNumber($data['number']);
        if(! $res) return [false,'请输入正确的必选产品种数'];

        if(empty($data['total_amount'])) return [false,'活动包总金额不能为空'];
        $res = $this->checkNumber($data['total_amount']);
        if(! $res) return [false,'活动包总金额请输入不超过两位小数并且大于0的数值'];

        //所属部门 以及  顶级部门
        if(empty($data['depart_id'])) $data['depart_id'] = $this->getDepart($user);
        $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;

        if($is_check){

        }else{
            if($this->isEmpty($data,'id')) return [false,'活动包ID不能为空！'];
        }

        return [true, ''];
    }
}
